package com.olimex.libi2c;

import com.olimex.libi2c.CommunicationException;

public class libI2C {

	public static int fd;
	public static int ADDRESS;
	public static String BUS;
	
	

	/**
	 * Open hardware i2c interface.
	 * 
	 * @param bus
	 *            Name of the device, e.g. "/dev/i2c-0", "/dev/i2c-1" or
	 *            "/dev/2c-2".
	 * @return File descriptor for the bus. If there is error returns -1.
	 */
	public static native int init(String bus);

	/**
	 * Connect to i2c device.
	 * 
	 * @param fd
	 *            File descriptor for the i2c bus.
	 * @param address
	 *            Address of the device.
	 * @return Return status of the operation: -1 - ERROR, 0 - OK.
	 */
	public static native int open(int fd, int address);

	/**
	 * Close i2c bus.
	 * 
	 * @param fd
	 *            File descriptor for the i2c bus.
	 * @return Return status of the operation: -1 - ERROR, 0 - OK.
	 */
	public static native int close(int fd);

	/**
	 * Send data to i2c device.
	 * 
	 * @param fd
	 *            File descriptor for the i2c bus.
	 * @param buf
	 *            Buffer containing data to be send.
	 * @param len
	 *            Number of byte to be send.
	 * @return Return status of the operation: -1 - ERROR, 0 - OK.
	 */
	public static native int write(int fd, int[] buf, int len);

	/**
	 * Read data from i2c device.
	 * 
	 * @param fd
	 *            File descriptor for the i2c bus.
	 * @param buf
	 *            Buffer that will contain read data.
	 * @param len
	 *            Number of bytes to be read.
	 * @return Return status of the operation: -1 - ERROR, 0 - OK.
	 */
	public static native int read(int fd, int[] buf, int len);

	/**
	 * Write data to register.
	 * 
	 * @param register
	 *            Point to the first register to write.
	 * @param data
	 *            Data to write. Length must be exact to the data to write.
	 * @throws CommunicationException
	 *             If there is error throws exception.
	 */
	public static void writeData(int register, int[] data)
			throws CommunicationException {

		int[] out_buffer = new int[data.length + 1];

		fd = init(BUS);
		if (fd < 0)
			throw CommunicationException.init_exception;

		if (open(fd, ADDRESS) < 0)
			throw CommunicationException.open_exception;

		out_buffer[0] = register;
		for (int i = 0; i < data.length; i++)
			out_buffer[i + 1] = data[i];
		if (write(fd, out_buffer, out_buffer.length) < 0)
			throw CommunicationException.write_exception;

		if (close(fd) < 0)
			throw CommunicationException.close_exception;
	}

	/**
	 * Read data from the board.
	 * 
	 * @param register
	 *            Point to the first register to read.
	 * @param len
	 *            Number of bytes to read.
	 * @return Return integer array with read data.
	 * @throws CommunicationException
	 *             If there is error throws exception.
	 */
	public static int[] readData(int register, int len)
			throws CommunicationException {
		int[] in_buffer = new int[len];

		fd = init(BUS);
		if (fd < 0)
			throw CommunicationException.init_exception;

		if (open(fd, ADDRESS) < 0)
			throw CommunicationException.open_exception;

		if (write(fd, new int[] { register }, 1) < 0)
			throw CommunicationException.write_exception;

		if (read(fd, in_buffer, len) < 0)
			throw CommunicationException.read_exception;

		if (close(fd) < 0)
			throw CommunicationException.close_exception;

		return in_buffer;
	}

	/**
	 * Load native shared library
	 */
	static {
		System.loadLibrary("i2c");
	}
}
